# ERMC SDK

The **ERMC SDK** provides methods to report bet events and withdrawal events with automatic location detection. This SDK supports both GPS geolocation and IP-based fallback, and can be used in both browser environments (via CDN) and Node.js environments (via npm package).

## Features

🌍 **Automatic Location Detection**

- GPS/Browser geolocation (primary)
- Multi-service IP geolocation fallback
- Custom location support
- Enhanced mobile device compatibility

🔒 **Robust & Secure**

- HTTPS-compatible IP services
- Graceful permission handling
- Error resilience with multiple fallback services
- Works in both HTTP and HTTPS contexts

📱 **Mobile Optimized**

- Android geolocation support
- Increased timeouts for mobile networks
- Permission denial handling

## Installation

### Using npm

To install the SDK via npm, run the following command:

```sh
npm install ermc-sdk
```

### Using CDN

To use the SDK via CDN, include the following script tag in your HTML file:

```html
<script src="https://cdn.jsdelivr.net/npm/ermc-sdk@1.1.1/dist/location-sdk.umd.js"></script>
```

## Usage

### Quick Start

```javascript
// Browser environment
const apiKey = "your-api-key";

// Report a bet event with automatic location detection
LocationSDK.reportBetEvent(apiKey, {
  bet_id: "bet_123",
  user_id: "user_456",
  stake_amount: 100,
  odds: 2.5,
  expected_payout: 250,
  metadata: { sport: "football" },
})
  .then((response) => {
    console.log("Bet event reported:", response);
    console.log("Location source:", response.location.source); // "gps" or "ip"
  })
  .catch((error) => {
    console.error("Error:", error);
  });

// Report with custom location (optional)
LocationSDK.reportBetEvent(apiKey, eventDetails, "Lagos, Nigeria");
```

### Location Detection Flow

1. **GPS First**: Attempts to get precise location via browser geolocation
2. **IP Fallback**: If GPS fails/denied, automatically uses IP geolocation
3. **Multiple IP Services**: Tries ipapi.co → ipinfo.io → freeipapi.com
4. **Custom Location**: Optional custom location string can be provided

### Methods

#### `LocationSDK.getLocation()`

Gets the user's current location with automatic GPS → IP fallback.

**Returns:**

- `Promise<Object>`: Location data with source indicator ("gps" or "ip")

```javascript
const location = await LocationSDK.getLocation();
console.log(location.source); // "gps" or "ip"
console.log(location.service); // IP service used (if applicable)
```

#### `LocationSDK.getLocationFromIP()`

Directly gets location from IP address using multiple fallback services.

**Returns:**

- `Promise<Object>`: IP-based location data with service indicator

#### `LocationSDK.attachLocation(eventDetails, customLocation)`

Attaches location data to the provided event details.

**Parameters:**

- `eventDetails` (`Object`): The original event details
- `customLocation` (`string`, optional): Custom location string (e.g., "Lagos")

**Returns:**

- `Promise<Object>`: Updated event details with location data

#### `LocationSDK.sendEvent(url, headers, data, customLocation)`

Sends a request to the specified URL with the provided headers and data.

**Parameters:**

- `url` (`string`): The endpoint URL
- `headers` (`Object`): The request headers
- `data` (`Object`): The request data
- `customLocation` (`string`, optional): Custom location string

**Returns:**

- `Promise<Object>`: The response data

#### `LocationSDK.reportBetEvent(apiKey, eventDetails, customLocation)`

Reports a bet event with automatic location detection.

**Parameters:**

- `apiKey` (`string`): The API key
- `eventDetails` (`Object`): The event details
  - `bet_id` (`string`): The bet ID
  - `user_id` (`string`): The user ID
  - `stake_amount` (`number`): The stake amount
  - `odds` (`number`): The odds
  - `expected_payout` (`number`): The expected payout
  - `metadata` (`Object`): Additional metadata
- `customLocation` (`string`, optional): Custom location string (e.g., "Lagos")

**Returns:**

- `Promise<Object>`: Response data with location information

#### `LocationSDK.reportWithdrawalEvent(apiKey, eventDetails, customLocation)`

Reports a withdrawal event with automatic location detection.

**Parameters:**

- `apiKey` (`string`): The API key
- `eventDetails` (`Object`): The event details
  - `user_id` (`string`): The user ID
  - `amount` (`number`): The amount
  - `request_id` (`string`): The request ID
  - `metadata` (`Object`): Additional metadata
- `customLocation` (`string`, optional): Custom location string (e.g., "Lagos")

**Returns:**

- `Promise<Object>`: Response data with location information

## Location Data Structure

The SDK automatically attaches location data in the following format:

```javascript
{
  // Your original event data
  bet_id: "bet_123",
  user_id: "user_456",
  // ... other fields

  // Automatically attached location data
  location: {
    latitude: 6.5244,
    longitude: 3.3792,
    accuracy: 65, // meters (GPS) or 10000 (IP)
    source: "gps", // "gps" or "ip"
    service: "ipapi.co", // Present for IP-based locations
    city: "Lagos",
    region: "Lagos",
    country: "Nigeria",
    timestamp: 1623456789000
  },

  // Optional custom location (if provided)
  custom_location: "Lagos, Nigeria"
}
```

## Browser Compatibility

- ✅ Chrome 50+
- ✅ Firefox 55+
- ✅ Safari 11+
- ✅ Edge 79+
- ✅ Mobile browsers (iOS Safari, Chrome Mobile)

## Error Handling

The SDK includes comprehensive error handling:

```javascript
try {
  const response = await LocationSDK.reportBetEvent(apiKey, eventDetails);
  // Success
} catch (error) {
  if (error.message.includes("Failed to get location")) {
    // Location detection failed
    console.log("Consider using custom location parameter");
  } else if (error.message.includes("Failed to send event")) {
    // API request failed
    console.log("Check API key and network connection");
  }
}
```

## Development

```bash
# Install dependencies
npm install

# Build the SDK
npm run build

# Built files will be in dist/
# - location-sdk.umd.js (for browsers)
# - location-sdk.esm.js (for ES modules)
```

## Changelog

### v1.1.1

- ❌ Removed geoplugin.net (requires paid subscription for HTTPS)
- ✅ Verified working with 3 free HTTPS services: ipapi.co, ipinfo.io, freeipapi.com

### v1.1.0

- 🔧 Fixed HTTPS compatibility for IP geolocation
- 🌐 Added multi-service IP fallback (ipapi.co, ipinfo.io, freeipapi.com)
- 📱 Improved mobile device support
- 🛡️ Enhanced error handling and permission management
- 📊 Added service detection for transparency
- ❌ Removed geoplugin.net (requires paid SSL access)

### v1.0.x

- Initial release with GPS and IP fallback
- Basic bet and withdrawal event reporting
- Custom location support
