const BASE_URL = "https://api.ermc.africa/integration-gateway/external";

class LocationSDK {
  /**
   * Checks if the current environment supports secure geolocation.
   * @returns {boolean}
   */
  static isSecureContext() {
    return (
      window.isSecureContext ||
      location.protocol === "https:" ||
      location.hostname === "localhost"
    );
  }

  /**
   * Gets location from IP address using free IP geolocation services.
   * Tries multiple services as fallbacks for reliability.
   * @returns {Promise<Object>} - A promise that resolves to location data from IP.
   */
  static async getLocationFromIP() {
    const services = [
      {
        name: "ipapi.co",
        url: "https://ipapi.co/json/",
        parser: (data) => ({
          latitude: parseFloat(data.latitude),
          longitude: parseFloat(data.longitude),
          accuracy: 10000,
          altitude: null,
          speed: null,
          timestamp: Date.now(),
          source: "ip",
          city: data.city,
          region: data.region,
          country: data.country_name,
          ip: data.ip,
          service: "ipapi.co",
        }),
      },
      {
        name: "ipinfo.io",
        url: "https://ipinfo.io/json",
        parser: (data) => {
          const [lat, lon] = (data.loc || "0,0").split(",");
          return {
            latitude: parseFloat(lat),
            longitude: parseFloat(lon),
            accuracy: 10000,
            altitude: null,
            speed: null,
            timestamp: Date.now(),
            source: "ip",
            city: data.city,
            region: data.region,
            country: data.country,
            ip: data.ip,
            service: "ipinfo.io",
          };
        },
      },
      {
        name: "freeipapi.com",
        url: "https://freeipapi.com/api/json",
        parser: (data) => ({
          latitude: parseFloat(data.latitude),
          longitude: parseFloat(data.longitude),
          accuracy: 10000,
          altitude: null,
          speed: null,
          timestamp: Date.now(),
          source: "ip",
          city: data.cityName,
          region: data.regionName,
          country: data.countryName,
          ip: data.ipAddress,
          service: "freeipapi.com",
        }),
      },
    ];

    let lastError = null;

    for (const service of services) {
      try {
        console.log(`Trying IP geolocation service: ${service.name}`);

        const response = await fetch(service.url, {
          method: "GET",
          headers: {
            Accept: "application/json",
          },
        });

        if (!response.ok) {
          throw new Error(
            `${service.name} error: ${response.statusText} (${response.status})`
          );
        }

        const data = await response.json();

        // Check for service-specific error formats
        if (data.error || (data.status && data.status === "fail")) {
          throw new Error(
            `${service.name} returned error: ${
              data.error || data.message || "Unknown error"
            }`
          );
        }

        const result = service.parser(data);

        // Validate the result
        if (
          !result.latitude ||
          !result.longitude ||
          isNaN(result.latitude) ||
          isNaN(result.longitude)
        ) {
          throw new Error(`${service.name} returned invalid coordinates`);
        }

        console.log(`Successfully got location from ${service.name}`);
        return result;
      } catch (error) {
        console.warn(`${service.name} failed:`, error.message);
        lastError = error;
        continue; // Try next service
      }
    }

    // If all services failed, throw the last error
    throw new Error(
      `All IP geolocation services failed. Last error: ${
        lastError?.message || "Unknown error"
      }`
    );
  }

  /**
   * Gets the current location using the browser's geolocation API.
   * Falls back to IP geolocation if GPS fails.
   * @returns {Promise<Object>} - A promise that resolves to the coordinates { latitude, longitude }.
   */
  static async getLocation() {
    // First try browser geolocation (most accurate)
    if (navigator.geolocation && this.isSecureContext()) {
      try {
        return await new Promise((resolve, reject) => {
          // Enhanced options for mobile devices, especially Android
          const options = {
            enableHighAccuracy: true, // Use GPS if available
            timeout: 30000, // Increased timeout for mobile devices
            maximumAge: 60000, // Cache location for 1 minute to improve performance
          };

          navigator.geolocation.getCurrentPosition(
            (position) => {
              resolve({
                latitude: position.coords.latitude,
                longitude: position.coords.longitude,
                accuracy: position.coords.accuracy,
                altitude: position.coords.altitude,
                speed: position.coords.speed,
                timestamp: position.timestamp,
                source: "gps", // Indicate this came from GPS/browser geolocation
              });
            },
            async (error) => {
              console.warn(
                `GPS geolocation failed (${error.code}: ${error.message}), falling back to IP geolocation`
              );
              try {
                const ipLocation = await this.getLocationFromIP();
                resolve(ipLocation);
              } catch (ipError) {
                reject(
                  new Error(
                    `Both GPS and IP geolocation failed. GPS: ${error.message}, IP: ${ipError.message}`
                  )
                );
              }
            },
            options
          );
        });
      } catch (gpsError) {
        console.warn(
          "GPS geolocation error, falling back to IP geolocation:",
          gpsError.message
        );
        return await this.getLocationFromIP();
      }
    } else {
      // Browser doesn't support geolocation or not in secure context, use IP fallback
      console.warn(
        "Browser geolocation not available or not in secure context, using IP geolocation"
      );
      return await this.getLocationFromIP();
    }
  }

  /**
   * Attaches location data to the provided event details.
   * @param {Object} eventDetails - The original event details.
   * @param {string} [customLocation] - Optional custom location string (like "Lagos").
   * @returns {Promise<Object>} - The updated event details with location data.
   */
  static async attachLocation(eventDetails, customLocation = null) {
    try {
      // Always try to get location (GPS first, then IP fallback)
      const coordinates = await this.getLocation();

      const result = {
        ...eventDetails,
        location: coordinates,
      };

      // If custom location is provided, add it as a separate field
      if (customLocation && typeof customLocation === "string") {
        result.custom_location = customLocation;
      }

      return result;
    } catch (error) {
      // If both GPS and IP geolocation fail but custom location is provided, use only custom location
      if (customLocation && typeof customLocation === "string") {
        return {
          ...eventDetails,
          custom_location: customLocation,
        };
      }
      throw new Error(`Failed to get location: ${error.message}`);
    }
  }

  /**
   * Sends a request to the specified URL with the provided headers and data.
   * @param {string} url - The endpoint URL.
   * @param {Object} headers - The request headers.
   * @param {Object} data - The request data.
   * @param {string} [customLocation] - Optional custom location string (like "Lagos").
   * @returns {Promise<Object>} - The response data.
   */
  static async sendEvent(url, headers, data, customLocation = null) {
    try {
      const updatedData = await this.attachLocation(data, customLocation);

      const response = await fetch(url, {
        method: "POST",
        headers,
        body: JSON.stringify(updatedData),
      });

      if (!response.ok) {
        throw new Error(`Failed to send event: ${response.statusText}`);
      }

      return await response.json();
    } catch (error) {
      throw new Error(`Error sending event: ${error.message}`);
    }
  }

  /**
   * Reports a bet event.
   * @param {string} apiKey - The API key.
   * @param {Object} eventDetails - The event details.
   * @param {string} eventDetails.user_id - The user ID.
   * @param {number} eventDetails.amount - The amount.
   * @param {string} eventDetails.bet_id - The bet ID.
   * @param {Object} eventDetails.metadata - Additional metadata.
   * @param {string} [customLocation] - Optional custom location string (like "Lagos").
   * @returns {Promise<Object>} - The response data.
   */
  static reportBetEvent(apiKey, eventDetails, customLocation = null) {
    const url = `${BASE_URL}/report-bet-event`;
    const headers = {
      "x-api-key": apiKey,
      "Content-Type": "application/json",
    };
    return this.sendEvent(url, headers, eventDetails, customLocation);
  }

  /**
   * Reports a withdrawal event.
   * @param {string} apiKey - The API key.
   * @param {Object} eventDetails - The event details.
   * @param {string} eventDetails.user_id - The user ID.
   * @param {number} eventDetails.amount - The amount.
   * @param {string} eventDetails.request_id - The request ID.
   * @param {Object} eventDetails.metadata - Additional metadata.
   * @param {string} [customLocation] - Optional custom location string (like "Lagos").
   * @returns {Promise<Object>} - The response data.
   */
  static reportWithdrawalEvent(apiKey, eventDetails, customLocation = null) {
    const url = `${BASE_URL}/report-withdrawal`;
    const headers = {
      "x-api-key": apiKey,
      "Content-Type": "application/json",
    };
    return this.sendEvent(url, headers, eventDetails, customLocation);
  }
}

// Export for both Node.js and browser environments
if (typeof module !== "undefined" && module.exports) {
  module.exports = LocationSDK;
} else if (typeof window !== "undefined") {
  window.LocationSDK = LocationSDK;
}

export default LocationSDK;
